import * as ___index from '../index/index.js';
import { Middleware } from '../index/index.js';

declare const immutable: Middleware;
declare const useSWRImmutable: ___index.SWRHook;

export { useSWRImmutable as default, immutable };
